#!/usr/bin/python

import json
#import logging
#logging.basicConfig(level=logging.DEBUG)
import os
import shutil
import tempfile
import unittest

from CommandNotFound.CommandNotFound import (
    CommandNotFound,
    SqliteDatabase,
)
from CommandNotFound.db.creator import DbCreator


class MockAptDB:
    def __init__(self, db):
        self._db = db
    def lookup(self, command):
        return self._db.get(command, [])

   
class CommandNotFoundOutputTest(unittest.TestCase):

    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()
        self.cnf = CommandNotFound()
        self.cnf.db = MockAptDB({
            'x-pylint': [("pylint", "1.0", "universe")],
            'x-vi': [("vim", "1.0", "main"),
                     ("neovim", "2.0", "universe"),
                    ],
            'x-aws': [("aws", "1.0", "main")],
        })
        self.cnf.output_fd = open(os.path.join(self.tmpdir, "output"), "w")
        self.user_can_sudo = True
        self.cnf.snap_cmd = os.path.join(self.tmpdir, "mock-snap-cmd")

    def tearDown(self):
        self.cnf.output_fd.close()
        shutil.rmtree(self.tmpdir)

    def set_mock_snap_cmd_json(self, json):
        with open(self.cnf.snap_cmd, "w") as fp:
            fp.write("""#!/bin/sh
set -e

echo '%s'
""" % json)
        os.chmod(self.cnf.snap_cmd, 0o755)

    def test_single_snap_from_spec(self):
        spec = """
# single snap advise
snaps: spotify:1.0:x-spotify
with: x-spotify

Command 'x-spotify' not found, but can be installed as:

sudo snap install spotify

"""
        self._test_spec(spec)

    def test_mixed_results_from_spec(self):
        spec = """
# mixed advise
debs: aws:1.0:x-aws,other-cmd
snaps: aws-cli:2.0:x-aws
with: x-aws

Command 'x-aws' not found, but can be installed as:

  aws-cli (2.0, snap)
  aws (1.0, deb)

See 'snap info <snap name>' for additional versions.

"""
        self._test_spec(spec)
        
    def _test_spec(self, spec):
        self.cnf.db = MockAptDB({})
        lines = spec.split("\n")
        for i, line in enumerate(lines):
            if line.startswith("debs: "):
                debs = line[len("debs: "):].split(";")
                for deb in debs:
                    l = deb.split(":")
                    pkgname = l[0]
                    ver = l[1]
                    cmds = l[2].split(",")
                    for cmd in cmds:
                        if not cmd in self.cnf.db._db:
                            self.cnf.db._db[cmd] = []
                        self.cnf.db._db[cmd].append( (pkgname, ver, "main") )
            if line.startswith("snaps: "):
                snaps = line[len("snaps: "):].split(";")
                mock_json = []
                for snap in snaps:
                    l = snap.split(":")
                    name = l[0]
                    ver = l[1]
                    cmds = l[2].split(",")
                    for cmd in cmds:
                        mock_json.append({"Snap": name, "Command": cmd, "Version": ver})
                self.set_mock_snap_cmd_json(json.dumps(mock_json))
            if line.startswith("with: "):
                cmd = line[len("with: "):]
                break
        expected_output = "\n".join(lines[i+1:])
        self.cnf.advise(cmd)
        with open(self.cnf.output_fd.name) as fp:
            output = fp.read()
        self.assertEqual(output, expected_output)
        
    def test_single_advise_deb(self):
        self.cnf.advise("x-pylint")
        with open(self.cnf.output_fd.name) as fp:
            output = fp.read()
        self.assertEqual(output, """

Command 'x-pylint' not found, but can be installed as:

sudo apt install pylint

"""[1:])

    def test_multi_advise_deb(self):
        self.cnf.advise("x-vi")
        with open(self.cnf.output_fd.name) as fp:
            output = fp.read()
        self.assertEqual(output, """

Command 'x-vi' not found, but can be installed as:

  vim (1.0, deb)
  neovim (2.0, deb)

Try: sudo apt install <deb name>

"""[1:])

    def test_multi_fuzzy_advise_deb(self):
        self.cnf.advise("x-via")
        with open(self.cnf.output_fd.name) as fp:
            output = fp.read()
        self.assertEqual(output, """

Command 'x-via' not found, did you mean:

  command 'x-vi' from deb vim (1.0)
  command 'x-vi' from deb neovim (2.0)

Try: sudo apt install <deb name>

"""[1:])

    def test_single_advise_snap(self):
        self.set_mock_snap_cmd_json(
            '[{"Snap": "spotify", "Command": "x-spotify", "Version": "1.0"}]'
        )
        self.cnf.advise("x-spotify")
        with open(self.cnf.output_fd.name) as fp:
            output = fp.read()
        self.assertEqual(output, """

Command 'x-spotify' not found, but can be installed as:

sudo snap install spotify

"""[1:])

    def test_multi_advise_snap(self):
        self.set_mock_snap_cmd_json(
            '[{"Snap": "foo1", "Command": "x-foo", "Version": "1.0"},'
            ' {"Snap": "foo2", "Command": "x-foo", "Version": "2.0"}]'
        )
        self.cnf.advise("x-foo")
        with open(self.cnf.output_fd.name) as fp:
            output = fp.read()
        self.assertEqual(output, """

Command 'x-foo' not found, but can be installed as:

  foo1 (1.0, snap)
  foo2 (2.0, snap)

See 'snap info <snap name>' for additional versions.

"""[1:])

    def test_mixed_advise(self):
        self.set_mock_snap_cmd_json(
            '[{"Snap": "aws-cli", "Command": "x-aws", "Version": "2.0"}]'
        )
        self.cnf.advise("x-aws")
        with open(self.cnf.output_fd.name) as fp:
            output = fp.read()
        self.assertEqual(output, """

Command 'x-aws' not found, but can be installed as:

  aws-cli (2.0, snap)
  aws (1.0, deb)

See 'snap info <snap name>' for additional versions.

"""[1:])

    def test_mixed_fuzzy_advise(self):
        self.set_mock_snap_cmd_json(
            '[{"Snap": "aws-cli", "Command": "x-aws", "Version": "2.0"}]'
        )
        self.cnf.advise("x-awsX")
        with open(self.cnf.output_fd.name) as fp:
            output = fp.read()
        self.assertEqual(output, """

Command 'x-awsX' not found, did you mean:

  command 'x-aws' from snap aws-cli (2.0)
  command 'x-aws' from deb aws (1.0)

See 'snap info <snap name>' for additional versions.

"""[1:])
        

class RegressionTestCase(unittest.TestCase):

    def test_lp1130444(self):
        tmpdir = tempfile.mkdtemp()
        self.addCleanup(shutil.rmtree, tmpdir)

        mock_commands_file = os.path.join(tmpdir, "Commands-all")
        with open(mock_commands_file, "w"):
            pass
        col = DbCreator([mock_commands_file])
        col.create(os.path.join(tmpdir, "test.db"))
        db = SqliteDatabase(os.path.join(tmpdir, "test.db"))
        self.assertEqual(db.lookup("foo\udcb6"), [])
